@@ -309,4 +309,30 @@ namespace zim
 
     return ret;
   }
-}
+
+  bool File::checkIntegrity(IntegrityCheck checkType)
+  {
+    return impl->checkIntegrity(checkType);
+  }
+
+  bool validate(const std::string& zimPath, IntegrityCheckList checksToRun)
+  {
+    try
+    {
+      File f(zimPath);
+      for ( size_t i = 0; i < checksToRun.size(); ++i )
+      {
+        if ( checksToRun.test(i) && !f.checkIntegrity(IntegrityCheck(i)) )
+          return false;
+      }
+    }
+    catch(ZimFileFormatError &exception)
+    {
+      std::cerr << exception.what() << std::endl;
+      return false;
+    }
+
+    return true;
+  }
+
+} // namespace zim
