@@ -22,29 +22,33 @@
 #include "creatordata.h"
 #include "cluster.h"
 #include "debug.h"
+#include "workers.h"
 #include <zim/blob.h>
 #include <zim/writer/creator.h>
 #include "../endian_tools.h"
 #include <algorithm>
 #include <fstream>
+#include "../md5.h"
 
 #if defined(ENABLE_XAPIAN)
   #include "xapianIndexer.h"
 #endif
 
 #ifdef _WIN32
-#include <io.h>
+# include <io.h>
 #else
-#include <unistd.h>
+# include <unistd.h>
+# define _write(fd, addr, size) if(::write((fd), (addr), (size)) != (ssize_t)(size)) \
+{throw std::runtime_error("Error writing");}
 #endif
 
+#include <sys/stat.h>
 #include <stdio.h>
+#include <fcntl.h>
 #include <limits>
 #include <stdexcept>
 #include <sstream>
 #include <ctime>
-#include "md5stream.h"
-#include "tee.h"
 #include "log.h"
 #include "../fs.h"
 #include "../tools.h"
@@ -64,63 +68,49 @@ log_define("zim.writer.creator")
                   << "; " << e << std::endl; \
     }
 
+#define CLUSTER_BASE_OFFSET 1024
+
 namespace zim
 {
   namespace writer
   {
-    void* Creator::clusterWriter(void* arg) {
-      auto creator = static_cast<zim::writer::Creator*>(arg);
-      zim::writer::Cluster* clusterToWrite;
-      unsigned int wait = 0;
-
-      while(true) {
-        microsleep(wait);
-        if (creator->data->clustersToWrite.popFromQueue(clusterToWrite)) {
-          wait = 0;
-          clusterToWrite->dump_tmp(creator->data->tmpfname);
-          clusterToWrite->close();
-          continue;
-        }
-        wait += 10;
-      }
-      return nullptr;
-    }
-
-    Creator::Creator(bool verbose)
+    Creator::Creator(bool verbose, CompressionType c)
       : verbose(verbose)
+      , compression(c)
     {}
 
     Creator::~Creator() = default;
 
     void Creator::startZimCreation(const std::string& fname)
     {
-      data = std::unique_ptr<CreatorData>(new CreatorData(fname, verbose, withIndex, indexingLanguage));
+      data = std::unique_ptr<CreatorData>(new CreatorData(fname, verbose, withIndex, indexingLanguage, compression));
       data->setMinChunkSize(minChunkSize);
 
-      for(unsigned i=0; i<compressionThreads; i++)
+      for(unsigned i=0; i<nbWorkerThreads; i++)
       {
         pthread_t thread;
-        pthread_create(&thread, NULL, clusterWriter, this);
-        pthread_detach(thread);
-        data->runningWriters.push_back(thread);
+        pthread_create(&thread, NULL, taskRunner, this->data.get());
+        data->workerThreads.push_back(thread);
       }
+
+      pthread_create(&data->writerThread, NULL, clusterWriter, this->data.get());
     }
 
-    void Creator::addArticle(const Article& article)
+    void Creator::addArticle(std::shared_ptr<Article> article)
     {
-      auto dirent = data->createDirentFromArticle(&article);
-      data->addDirent(dirent, &article);
+      auto dirent = data->createDirentFromArticle(article.get());
+      data->addDirent(dirent, article.get());
       data->nbArticles++;
-      if (article.isRedirect()) {
+      if (article->isRedirect()) {
         data->nbRedirectArticles++;
       } else {
-        if (article.shouldCompress())
+        if (article->shouldCompress())
           data->nbCompArticles++;
         else
           data->nbUnCompArticles++;
-        if (!article.getFilename().empty())
+        if (!article->getFilename().empty())
           data->nbFileArticles++;
-        if (article.shouldIndex())
+        if (article->shouldIndex())
           data->nbIndexArticles++;
       }
       if (verbose && data->nbArticles%1000 == 0){
@@ -135,13 +125,16 @@ namespace zim
                   << "; C:" << data->nbClusters
                   << "; CC:" << data->nbCompClusters
                   << "; UC:" << data->nbUnCompClusters
-                  << "; WC:" << data->clustersToWrite.size()
+                  << "; WC:" << data->taskList.size()
                   << std::endl;
       }
 
 #if defined(ENABLE_XAPIAN)
-      if(withIndex && article.shouldIndex()) {
-        data->indexer->index(&article);
+      if (article->shouldIndex()) {
+        data->titleIndexer.index(article.get());
+        if(withIndex && !article->isRedirect()) {
+          data->taskList.pushToQueue(new IndexTask(article));
+        }
       }
 #endif
     }
@@ -160,18 +153,39 @@ namespace zim
                   << "; C:" << data->nbClusters
                   << "; CC:" << data->nbCompClusters
                   << "; UC:" << data->nbUnCompClusters
-                  << "; WC:" << data->clustersToWrite.size()
+                  << "; WC:" << data->taskList.size()
                   << std::endl;
       }
 
+      // We need to wait that all indexation task has been done before closing the
+      // xapian database and add it to zim.
+      unsigned int wait = 0;
+      do {
+        microsleep(wait);
+        wait += 10;
+      } while(IndexTask::waiting_task.load() > 0);
+
 #if defined(ENABLE_XAPIAN)
+      {
+        data->titleIndexer.indexingPostlude();
+        auto article = data->titleIndexer.getMetaArticle();
+        auto dirent = data->createDirentFromArticle(article);
+        data->addDirent(dirent, article);
+        delete article;
+      }
       if (withIndex) {
-          data->indexer->indexingPostlude();
-          microsleep(100);
-          auto article = data->indexer->getMetaArticle();
-          auto dirent = data->createDirentFromArticle(article);
-          data->addDirent(dirent, article);
-          delete article;
+        wait = 0;
+        do {
+          microsleep(wait);
+          wait += 10;
+        } while(IndexTask::waiting_task.load() > 0);
+
+        data->indexer->indexingPostlude();
+        microsleep(100);
+        auto article = data->indexer->getMetaArticle();
+        auto dirent = data->createDirentFromArticle(article);
+        data->addDirent(dirent, article);
+        delete article;
       }
 #endif
 
@@ -182,41 +196,25 @@ namespace zim
       if (data->uncompCluster->count())
         data->closeCluster(false);
 
-      // wait all cluster writing has been done
-      unsigned int wait = 0;
-      do {
-        microsleep(wait);
-        wait += 10;
-      } while(!data->clustersToWrite.isEmpty());
-
-      // Be sure that all cluster are closed
+      TINFO("Waiting for workers");
+      // wait all cluster compression has been done
       wait = 0;
-      bool closed = true;
       do {
-        closed = true;
         microsleep(wait);
         wait += 10;
-        for(auto cluster: data->clustersList) {
-          if (!cluster->isClosed()) {
-            closed = false;
-            break;
-          }
-        }
-      } while(!closed);
+      } while(ClusterTask::waiting_task.load() > 0);
 
-// [FIXME] pthread_cancel is not defined in android NDK.
-// As we don't create zim on android platform, 
-// let's simply skip this code to still allow
-// compilation of libzim on android.
-#if !defined(__ANDROID__)
-      for(auto& thread: data->runningWriters)
-      {
-        pthread_cancel(thread);
+      // Quit all workerThreads
+      for (auto i=0U; i< nbWorkerThreads; i++) {
+        data->taskList.pushToQueue(nullptr);
+      }
+      for(auto& thread: data->workerThreads) {
+        pthread_join(thread, nullptr);
       }
-#endif
 
-      TINFO("Generate cluster offsets");
-      data->generateClustersOffsets();
+      // Wait for writerThread to finish.
+      data->clusterToWrite.pushToQueue(nullptr);
+      pthread_join(data->writerThread, nullptr);
 
       TINFO("ResolveRedirectIndexes");
       data->resolveRedirectIndexes();
@@ -230,26 +228,23 @@ namespace zim
       TINFO("create title index");
       data->createTitleIndex();
       TINFO(data->dirents.size() << " title index created");
-      TINFO(data->clusterOffsets.size() << " clusters created");
+      TINFO(data->clustersList.size() << " clusters created");
 
-      TINFO("fill header");
-      Fileheader header;
-      fillHeader(&header);
+      TINFO("write zimfile :");
+      write();
+      ::close(data->out_fd);
 
-      TINFO("write zimfile");
-      write(header, data->basename + ".zim.tmp");
-      zim::DEFAULTFS::rename(data->basename + ".zim.tmp", data->basename + ".zim");
+      TINFO("rename tmpfile to final one.");
+      DEFAULTFS::rename(data->basename+".zim.tmp", data->basename+".zim");
 
       TINFO("finish");
     }
 
-    void Creator::fillHeader(Fileheader* header)
+    void Creator::fillHeader(Fileheader* header) const
     {
       auto mainUrl = getMainUrl();
       auto layoutUrl = getLayoutUrl();
 
-      log_debug("main url=" << mainUrl << " layout url=" << layoutUrl);
-
       if (data->isExtended) {
         header->setMajorVersion(Fileheader::zimExtendedMajorVersion);
       } else {
@@ -265,13 +260,11 @@ namespace zim
         {
           if (mainUrl == dirent->getFullUrl())
           {
-            log_debug("main idx=" << dirent->getIdx());
             header->setMainPage(article_index_type(dirent->getIdx()));
           }
 
           if (layoutUrl == dirent->getFullUrl())
           {
-            log_debug("layout idx=" << dirent->getIdx());
             header->setLayoutPage(article_index_type(dirent->getIdx()));
           }
         }
@@ -280,131 +273,134 @@ namespace zim
       header->setUuid( getUuid() );
       header->setArticleCount( data->dirents.size() );
 
-      offset_type offset(Fileheader::size);
-      header->setMimeListPos( offset );
+      header->setMimeListPos( Fileheader::size );
 
-      offset += data->mimeListSize().v;
-      header->setUrlPtrPos( offset );
-
-      offset += data->urlPtrSize().v;
-      header->setTitleIdxPos( offset );
-      header->setClusterCount( data->clusterOffsets.size() );
-
-      offset += data->titleIdxSize().v + data->indexSize().v;
-      header->setClusterPtrPos( offset );
-
-      offset += data->clusterPtrSize().v + data->clustersSize.v;
-      header->setChecksumPos( offset );
+      header->setClusterCount( data->clustersList.size() );
     }
 
-    void Creator::write(const Fileheader& header, const std::string& fname) const
+    void Creator::write() const
     {
-      std::ofstream zimfile(fname);
-      Md5stream md5;
-      Tee out(zimfile, md5);
+      Fileheader header;
+      fillHeader(&header);
 
-      out << header;
+      int out_fd = data->out_fd;
 
-      log_debug("after writing header - pos=" << zimfile.tellp());
-
-      // write mime type list
+      lseek(out_fd, header.getMimeListPos(), SEEK_SET);
+      TINFO(" write mimetype list");
       for(auto& mimeType: data->mimeTypesList)
       {
-        out << mimeType << '\0';
+        _write(out_fd, mimeType.c_str(), mimeType.size()+1);
       }
 
-      out << '\0';
+      _write(out_fd, "", 1);
 
-      // write url ptr list
-      offset_t off(header.getTitleIdxPos() + data->titleIdxSize().v);
+      ASSERT(lseek(out_fd, 0, SEEK_CUR), <, CLUSTER_BASE_OFFSET);
+
+      TINFO(" write directory entries");
+      lseek(out_fd, 0, SEEK_END);
+      for (Dirent* dirent: data->dirents)
+      {
+        dirent->setOffset(offset_t(lseek(out_fd, 0, SEEK_CUR)));
+        dirent->write(out_fd);
+      }
+
+      TINFO(" write url prt list");
+      header.setUrlPtrPos(lseek(out_fd, 0, SEEK_CUR));
       for (auto& dirent: data->dirents)
       {
         char tmp_buff[sizeof(offset_type)];
-        toLittleEndian(off.v, tmp_buff);
-        out.write(tmp_buff, sizeof(offset_type));
-        off += dirent->getDirentSize();
+        toLittleEndian(dirent->getOffset(), tmp_buff);
+        _write(out_fd, tmp_buff, sizeof(offset_type));
       }
 
-      log_debug("after writing direntPtr - pos=" << out.tellp());
-
-      // write title index
+      TINFO(" write title index");
+      header.setTitleIdxPos(lseek(out_fd, 0, SEEK_CUR));
       for (Dirent* dirent: data->titleIdx)
       {
         char tmp_buff[sizeof(article_index_type)];
         toLittleEndian(dirent->getIdx().v, tmp_buff);
-        out.write(tmp_buff, sizeof(article_index_type));
+        _write(out_fd, tmp_buff, sizeof(article_index_type));
       }
 
-      log_debug("after writing fileIdxList - pos=" << out.tellp());
-
-      // write directory entries
-      for (Dirent* dirent: data->dirents)
+      TINFO(" write cluster offset list");
+      header.setClusterPtrPos(lseek(out_fd, 0, SEEK_CUR));
+      for (auto cluster : data->clustersList)
       {
-        out << *dirent;
-        log_debug("write " << dirent->getTitle() << " dirent.size()=" << dirent->getDirentSize() << " pos=" << out.tellp());
-      }
-
-      log_debug("after writing dirents - pos=" << out.tellp());
-
-      // write cluster offset list
-      off += data->clusterPtrSize();
-      for (auto clusterOffset : data->clusterOffsets)
-      {
-        offset_t o(off + clusterOffset);
         char tmp_buff[sizeof(offset_type)];
-        toLittleEndian(o.v, tmp_buff);
-        out.write(tmp_buff, sizeof(offset_type));
+        toLittleEndian(cluster->getOffset(), tmp_buff);
+        _write(out_fd, tmp_buff, sizeof(offset_type));
       }
 
-      log_debug("after writing clusterOffsets - pos=" << out.tellp());
+      header.setChecksumPos(lseek(out_fd, 0, SEEK_CUR));
 
-      // write cluster data
-      if (!data->isEmpty)
-      {
-        for(auto& cluster: data->clustersList)
-        {
-          ASSERT(cluster->isClosed(), ==, true);
-          cluster->write_final(out);
-        }
+      TINFO(" write header");
+      lseek(out_fd, 0, SEEK_SET);
+      header.write(out_fd);
+
+      TINFO(" write checksum");
+      struct zim_MD5_CTX md5ctx;
+      unsigned char batch_read[1024+1];
+      lseek(out_fd, 0, SEEK_SET);
+      zim_MD5Init(&md5ctx);
+      while (true) {
+         auto r = read(out_fd, batch_read, 1024);
+         if (r == -1) {
+           perror("Cannot read");
+           throw std::runtime_error("oups");
+         }
+         if (r == 0)
+           break;
+         batch_read[r] = 0;
+         zim_MD5Update(&md5ctx, batch_read, r);
       }
-      else
-        log_warn("no data found");
-
-      if (!out)
-        throw std::runtime_error("failed to write zimfile");
-
-      log_debug("after writing clusterData - pos=" << out.tellp());
       unsigned char digest[16];
-      md5.getDigest(digest);
-      zimfile.write(reinterpret_cast<const char*>(digest), 16);
+      zim_MD5Final(digest, &md5ctx);
+      _write(out_fd, reinterpret_cast<const char*>(digest), 16);
     }
 
     CreatorData::CreatorData(const std::string& fname,
                                    bool verbose,
                                    bool withIndex,
-                                   std::string language)
-      : withIndex(withIndex),
+                                   std::string language,
+                                   CompressionType c)
+      : compression(c),
+        withIndex(withIndex),
         indexingLanguage(language),
+#if defined(ENABLE_XAPIAN)
+        titleIndexer(language, IndexingMode::TITLE, true),
+#endif
         verbose(verbose),
         nbArticles(0),
         nbRedirectArticles(0),
         nbCompArticles(0),
-	nbUnCompArticles(0),
-	nbFileArticles(0),
-	nbIndexArticles(0),
-	nbClusters(0),
-	nbCompClusters(0),
-	nbUnCompClusters(0),
+        nbUnCompArticles(0),
+        nbFileArticles(0),
+        nbIndexArticles(0),
+        nbClusters(0),
+        nbCompClusters(0),
+        nbUnCompClusters(0),
         start_time(time(NULL))
     {
       basename =  (fname.size() > 4 && fname.compare(fname.size() - 4, 4, ".zim") == 0)
                         ? fname.substr(0, fname.size() - 4)
                         : fname;
-      tmpfname = basename + ".tmp";
-      if(!DEFAULTFS::makeDirectory(tmpfname)) {
-        throw std::runtime_error(
-          std::string("failed to create temporary directory ")
-        + tmpfname);
+      auto zim_name = basename + ".zim.tmp";
+#ifdef _WIN32
+int mode =  _S_IREAD | _S_IWRITE;
+#else
+      mode_t mode = S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH;
+#endif
+      out_fd = open(zim_name.c_str(), O_RDWR|O_CREAT|O_TRUNC, mode);
+      if (out_fd == -1){
+        perror(nullptr);
+        std::ostringstream ss;
+        ss << "Cannot create file " << zim_name;
+        throw std::runtime_error(ss.str());
+      }
+      if(lseek(out_fd, CLUSTER_BASE_OFFSET, SEEK_SET) != CLUSTER_BASE_OFFSET) {
+        close(out_fd);
+        perror(nullptr);
+        throw std::runtime_error("Impossible to seek in file");
       }
 
       // We keep both a "compressed cluster" and an "uncompressed cluster"
@@ -415,9 +411,10 @@ namespace zim
       uncompCluster = new Cluster(zimcompNone);
 
 #if defined(ENABLE_XAPIAN)
+      titleIndexer.indexingPrelude(basename+"_title.idx");
       if (withIndex) {
-          indexer = new XapianIndexer(indexingLanguage, true);
-          indexer->indexingPrelude(tmpfname+".idx");
+          indexer = new XapianIndexer(indexingLanguage, IndexingMode::FULL, true);
+          indexer->indexingPrelude(basename+".idx");
       }
 #endif
     }
@@ -431,10 +428,6 @@ namespace zim
       for(auto& cluster: clustersList) {
         delete cluster;
       }
-#ifndef _WIN32
-//[TODO] Implement remove for windows
-      DEFAULTFS::remove(tmpfname);
-#endif
 #if defined(ENABLE_XAPIAN)
       if (indexer)
         delete indexer;
@@ -451,7 +444,7 @@ namespace zim
           dirents.erase(ret.first);
           dirents.insert(dirent);
         } else {
-          std::cerr << "Impossible to add " << dirent->getFullUrl() << std::endl;
+          std::cerr << "Impossible to add " << dirent->getFullUrl().getLongUrl() << std::endl;
           std::cerr << "  dirent's title to add is : " << dirent->getTitle() << std::endl;
           std::cerr << "  existing dirent's title is : " << existing->getTitle() << std::endl;
           return;
@@ -504,13 +497,10 @@ namespace zim
       dirent->setUrl(article->getUrl());
       dirent->setTitle(article->getTitle());
 
-      log_debug("article " << dirent->getLongUrl() << " fetched");
-
       if (article->isRedirect())
       {
         dirent->setRedirect(nullptr);
         dirent->setRedirectUrl(article->getRedirectUrl());
-        log_debug("is redirect to " << dirent->getRedirectUrl());
       }
       else if (article->isLinktarget())
       {
@@ -524,11 +514,10 @@ namespace zim
       {
         auto mimetype = article->getMimeType();
         if (mimetype.empty()) {
-          std::cerr << "Warning, " << article->getUrl() << " have empty mimetype." << std::endl;
+          std::cerr << "Warning, " << article->getUrl().getLongUrl() << " have empty mimetype." << std::endl;
           mimetype = "application/octet-stream";
         }
         dirent->setMimeType(getMimeTypeIdx(mimetype));
-        log_debug("is article; mimetype " << dirent->getMimeType());
       }
       return dirent;
     }
@@ -547,9 +536,9 @@ namespace zim
       }
       cluster->setClusterIndex(cluster_index_t(clustersList.size()));
       clustersList.push_back(cluster);
-      clustersToWrite.pushToQueue(cluster);
+      taskList.pushToQueue(new ClusterTask(cluster));
+      clusterToWrite.pushToQueue(cluster);
 
-      log_debug("cluster written");
       if (cluster->is_extended() )
         isExtended = true;
       if (compressed)
@@ -561,16 +550,6 @@ namespace zim
       return cluster;
     }
 
-    void CreatorData::generateClustersOffsets()
-    {
-      clustersSize = zsize_t(0);
-      for(auto& cluster: clustersList)
-      {
-        clusterOffsets.push_back(offset_t(clustersSize.v));
-        clustersSize += cluster->getFinalSize();
-      }
-    }
-
     void CreatorData::setArticleIndexes()
     {
       // set index
@@ -591,7 +570,7 @@ namespace zim
         Dirent tmpDirent(dirent->getRedirectUrl());
         auto target_pos = dirents.find(&tmpDirent);
         if(target_pos == dirents.end()) {
-          INFO("Invalid redirection " << dirent->getUrl() << " redirecting to (missing) " << dirent->getRedirectUrl());
+          INFO("Invalid redirection " << dirent->getFullUrl().getLongUrl() << " redirecting to (missing) " << dirent->getRedirectUrl().getLongUrl());
           dirents.erase(dirent);
         } else  {
           dirent->setRedirect(*target_pos);
@@ -658,24 +637,5 @@ namespace zim
         throw std::runtime_error("mime type index not found");
       return it->second;
     }
-
-    zsize_t CreatorData::mimeListSize() const
-    {
-      size_type ret = 1;
-      for (auto& rmimeType: rmimeTypesMap)
-        ret += (rmimeType.second.size() + 1);
-      return zsize_t(ret);
-    }
-
-    zsize_t CreatorData::indexSize() const
-    {
-      size_type s = 0;
-
-      for (auto& dirent: dirents)
-        s += dirent->getDirentSize();
-
-      return zsize_t(s);
-    }
-
   }
 }
