@@ -34,7 +34,8 @@ class Queue {
         virtual bool isEmpty();
         virtual size_t size();
         virtual void pushToQueue(const T& element);
-        virtual bool popFromQueue(T &filename);
+        virtual bool getHead(T &element);
+        virtual bool popFromQueue(T &element);
 
     protected:
         std::queue<T>   m_realQueue;
@@ -80,6 +81,18 @@ void Queue<T>::pushToQueue(const T &element) {
     pthread_mutex_unlock(&m_queueMutex);
 }
 
+template<typename T>
+bool Queue<T>::getHead(T &element) {
+  pthread_mutex_lock(&m_queueMutex);
+  if (m_realQueue.empty()) {
+    pthread_mutex_unlock(&m_queueMutex);
+    return false;
+  }
+  element = m_realQueue.front();
+  pthread_mutex_unlock(&m_queueMutex);
+  return true;
+}
+
 template<typename T>
 bool Queue<T>::popFromQueue(T &element) {
     pthread_mutex_lock(&m_queueMutex);
